<?php
/*
 +=====================================================================+
 |     _   _ _        _       ____                                     |
 |    | \ | (_)_ __  (_) __ _/ ___|  ___ __ _ _ __  _ __   ___ _ __    |
 |    |  \| | | '_ \ | |/ _` \___ \ / __/ _` | '_ \| '_ \ / _ \ '__|   |
 |    | |\  | | | | || | (_| |___) | (_| (_| | | | | | | |  __/ |      |
 |    |_| \_|_|_| |_|/ |\__,_|____/ \___\__,_|_| |_|_| |_|\___|_|      |
 |                 |__/                                                |
 |                                                                     |
 | (c) NinTechNet ~ https://nintechnet.com/                            |
 +=====================================================================+ // sa+i18n
*/

if (! defined( 'ABSPATH' ) ) { die( 'Forbidden' ); }

// In a multi-site environment, we load the code only for
// the superadmin and from the network admin screen only.
if ( is_multisite() && ! is_main_site() ) { return; }

$nscan_options = get_option( 'nscan_options' );

// AJAX security nonce:
$nscan_nonce = wp_create_nonce( 'nscan_ajax' );

// Add a "Summary" page link to the status span:
if (! empty( $nscan_options['scan_toolbarintegration']) ) {
	if ( is_multisite() ) {	$net = 'network/'; } else { $net = '';	}
	$summary_page = get_admin_url( null, "{$net}admin.php?page=NinjaScanner&nscantab=summary" );
}

$nscan_is_running = 0;
// Check the lock status:
if ( nscan_is_scan_running() !== false ) {
	$nscan_is_running = 1;
}
// Check if the scanning process transient is set:
if ( get_transient( 'nscan_start' ) !== false ) {
	delete_transient( 'nscan_start' );
	$nscan_is_running = 1;
}

?>
<script type="text/javascript" >

var nscan_interval = 0;
var nscan_error = 0;
var nscan_milliseconds = <?php echo (int) NSCAN_MILLISECONDS ?>;

jQuery(document).ready(function($) {

	<?php
	// Display status in the toolbar if the scan is running and the user enabled that option:
	if ( $nscan_is_running && ! empty( $nscan_options['scan_toolbarintegration']) ) {
		?>
		if (! $('#wp-admin-bar-nscan_status').length ) {
			jQuery('#wp-admin-bar-root-default').prepend('<li id="wp-admin-bar-nscan_status"><a class="ab-item" href="<?php echo $summary_page ?>"><span id="nscan-status-span" style="background-color:orange;padding:4px;border-radius:2px;color:white;box-shadow:1px 1px 2px #7F0000 inset;">&nbsp;<?php echo esc_js( __('Scan in progress...', 'ninjascanner' ) ) ?>&nbsp;</span></a></li>');
		}
		<?php
	}

	// If a scan is running, probe it every 5 seconds:
	if ( $nscan_is_running ) {
		?>
		nscan_check_if_running();
		nscan_interval = setInterval( nscan_check_if_running, nscan_milliseconds );
		<?php
	}
	?>

	function nscan_check_if_running() {

		var data = {
			'action': 'nscan_ajax',
			'nscan_nonce': '<?php echo $nscan_nonce; ?>'
		};
		jQuery.post(ajaxurl, data, function(response) {

			var res = response.split(':');

			// Scan is still running:
			if ( res[0] == 'running' ) {
				<?php
				if (! empty( $nscan_options['scan_toolbarintegration']) ) {
					// Show status in the toolbar (can be turned off in the Settings page):
					?>
					if (! $('#wp-admin-bar-nscan_status').length ) {
						$('#wp-admin-bar-root-default').prepend('<li id="wp-admin-bar-nscan_status"><a class="ab-item" href="<?php echo $summary_page ?>"><span id="nscan-status-span" style="background-color:orange;padding:4px;border-radius:2px;color:white;box-shadow:1px 1px 2px #7F0000 inset;">&nbsp;<?php echo esc_js( __('Scan in progress...', 'ninjascanner' ) ) ?>&nbsp;</span></a></li>');
					}
					<?php
				}
				?>
				// Display progress bar:
				var steps = 10;
				var percent = parseInt( res[1] * (100 / steps) );
				$('#ns-span-progress').css('width', percent + '%');
				$('#ns-div-progress').html( '<?php echo esc_js( __('Step', 'ninjascanner' ) ) ?> ' + res[1] + '/' + steps );

			// Scan has finished:
			} else if  ( res[0] == 'stopped' ) {
				clearInterval( nscan_interval );
				location.reload();

			// Too many AJAX errors? Clear the timer and display
			// an error message in the toolbar:
			} else {
				++nscan_error;
				if ( nscan_error > 10 ) {
					<?php
					if (! empty( $nscan_options['scan_toolbarintegration']) ) {
						?>
						if ( $('#wp-admin-bar-nscan_status').length ) {
							$('#wp-admin-bar-nscan_status').html( '<a class="ab-item" href="<?php echo $summary_page ?>"><span id="nscan-status-span-pulse" style="background-color:#FF0000;padding:4px;border-radius:2px;color:white;box-shadow:1px 1px 2px #7F0000 inset;">&nbsp;<?php echo esc_js( __('Scan error!', 'ninjascanner' ) ) ?>&nbsp;</span></a>' );
						}
						<?php
					}
					?>
					clearInterval( nscan_interval );
				}
			}
		});
	}

	// Cancel scan button clicked:
	$('#cancel-scan').click(function (e) {
		if ( confirm( nscani18n.cancel_scan ) ) {
			// Clear timer:
			if ( nscan_interval != 0 ) {
				clearInterval( nscan_interval );
			}
			// Reload the page (form submission):
			return true;
		}
		return false;
	});
});

</script>
