<?php
/*
 +=====================================================================+
 |     _   _ _        _       ____                                     |
 |    | \ | (_)_ __  (_) __ _/ ___|  ___ __ _ _ __  _ __   ___ _ __    |
 |    |  \| | | '_ \ | |/ _` \___ \ / __/ _` | '_ \| '_ \ / _ \ '__|   |
 |    | |\  | | | | || | (_| |___) | (_| (_| | | | | | | |  __/ |      |
 |    |_| \_|_|_| |_|/ |\__,_|____/ \___\__,_|_| |_|_| |_|\___|_|      |
 |                 |__/                                                |
 |                                                                     |
 | (c) NinTechNet ~ https://nintechnet.com/                            |
 +=====================================================================+ // sa+i18n
*/
if (! defined( 'ABSPATH' ) ) { die( 'Forbidden' ); }

// =====================================================================
// Display the scan report in HTML format.

function html_report() {

	$report = array();
	$snapshot = array();

	// Make sure we have a snapshot:
	if (! file_exists( NSCAN_SNAPSHOT ) ) {
		$report['error'] = __("Cannot find the snapshot file, scan report cannot be created.", 'ninjascanner');
		return $report;
	}

	if ( nscan_is_json_encoded( NSCAN_SNAPSHOT ) === true ) {
		$snapshot = json_decode( file_get_contents( NSCAN_SNAPSHOT ), true );
	} else {
		$snapshot = unserialize( file_get_contents( NSCAN_SNAPSHOT ) );
	}
	if ( empty( $snapshot['abspath'] ) ) {
		$report['error'] = __("Snapshot seems corrupted (missing 'abspath' field), scan report cannot be created.", 'ninjascanner');
		return $report;
	}

	$report['body'] = '';

	// Make sure the snapshot was created from the same WP version:
	global $wp_version;
	if ( $snapshot['version'] != $wp_version ) {
		$message = sprintf(
			__("This report was created for WordPress %s, but your current version is %s. ".
				"If you upgraded WordPress lately, don't forget to run a new scan.", 'ninjascanner'),
			htmlspecialchars( $snapshot['version'] ),
			htmlspecialchars( $wp_version )
		);
		echo '<div class="notice-warning notice is-dismissible"><p>' . $message . '</p></div>';
	}

	// Make sure it was created with the same version of NinjaScanner
	// (newer updates may bring new features - or remove some):
	if ( empty( $snapshot['nscan_version'] ) || $snapshot['nscan_version'] !== NSCAN_VERSION ) {
		$message = __("This report was created with a different version of NinjaScanner. ".
					"Don't forget to run a new scan to make sure it is up to date.", 'ninjascanner');
		echo '<div class="notice-warning notice is-dismissible"><p>' . $message . '</p></div>';
	}

	// Blog domain name:
	if ( is_multisite() ) {
		$blog = network_home_url('/');
	} else {
		$blog = home_url('/');
	}

	// Scan date:
	nscan_get_blogtimezone();
	$scan_date = ucfirst( date_i18n( 'F d, Y @ g:i A', filemtime( NSCAN_SNAPSHOT ) ) );
	if (! empty( $snapshot['locale'] ) ) {
		$wordpress = "{$snapshot['version']} ({$snapshot['locale']})";
	} else {
		$wordpress = "{$snapshot['version']}";
	}
	$report['body'] .= '<h3>'. __('NinjaScanner report', 'ninjascanner') .'</h3>
	<table class="widefat">
		<tr>
			<td class="r1">'. __('Date', 'ninjascanner') .'</td>
			<td class="r2">'. htmlentities( $scan_date ) .'</td>
		</tr>
		<tr>
			<td class="r1">'. __('Home URL', 'ninjascanner') .'</td>
			<td class="r2">'. htmlentities( $blog ) .'</td>
		</tr>
		<tr>
			<td class="r1">'. __('Blog directory', 'ninjascanner') .'</td>
			<td class="r2">'. htmlentities( ABSPATH ) .'</td>
		</tr>
		<tr>
			<td class="r1">'. __('WordPress Version', 'ninjascanner') .'</td>
			<td class="r2">'. htmlentities( $wordpress ) .'</td>
		</tr>
		<tr>
			<td class="r1">'. __('Total files', 'ninjascanner') .'</td>
			<td class="r2">'. number_format_i18n( count( $snapshot['abspath'] ) ) .'</td>
		</tr>
		<tr>
			<td class="r1">'. __('Legend', 'ninjascanner') .'</td>
			<td class="r2">
				<span class="nscan-ok">&nbsp;'. __('OK', 'ninjascanner') .'&nbsp;</span>&nbsp;&nbsp;
				<span class="nscan-notice">&nbsp;'. __('Notice', 'ninjascanner') .'&nbsp;</span>&nbsp;&nbsp;
				<span class="nscan-warning">&nbsp;'. __('Important', 'ninjascanner') .'&nbsp;</span>&nbsp;&nbsp;
				<span class="nscan-critical">&nbsp;'. __('Critical', 'ninjascanner') .'&nbsp;</span>
			</td>
		</tr>
	</table>
	<br />';


	// ==================================================================
	// WordPress core files integrity
	// Mismatched checksums:
	$report['body'] .= '<h3>'. __('WordPress core files integrity', 'ninjascanner') .'</h3>
	<table class="widefat">
		<tr>
			<td class="r1"><p>'. __('WordPress', 'ninjascanner') .'</p></td>
			<td class="r2">';

	// Error?
	if (! empty( $snapshot['step_error'][3] ) ) {
		$report['body'] .= '<p><span class="nscan-critical">&nbsp;'. sprintf(
			__('Warning, a critical error occurred. This test was cancelled: %s', 'ninjascanner'),
			$snapshot['step_error'][3]
		) ."&nbsp;</span></p>";

	} elseif (! empty( $snapshot['skip']['scan_wpcoreintegrity'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		if (! empty( $snapshot['core_failed_checksum'] ) ) {
			$count = count( $snapshot['core_failed_checksum'] );
			$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Modified core files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __('The following files do not match the original WordPress core files.', 'ninjascanner');
			// Inform users that they can repair their broken WP with a one-click button:
			if ( is_multisite() ) {	$net = 'network/'; } else { $net = '';	}
			$get_admin_url = get_admin_url( null, "{$net}update-core.php" );
			$text .= ' '. __('If they are damaged or corrupted, you can easily repair them by selecting a file and clicking on the "Restore file" button', 'ninjascanner' ) .'.';
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-1" onChange="nscan_file_info(this.value,1);" style="width:100%;" size="'. select_size($count) .'">';
			foreach( $snapshot['core_failed_checksum'] as $file => $null ) {
				$relative = str_replace( ABSPATH, '', $file );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(1, 'view', 'compare', 'restore');
		} else {
			$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
		}
		$report['body'] .= '
				</td>
			</tr>';

		// Unknown and suspicious files (Critical level):
		if (! empty( $snapshot['core_unknown'] ) ) {
			$report['body'] .= '
			<tr>
				<td class="r1"><p>'. __('Suspicious files', 'ninjascanner') .'</p></td>
				<td class="r2">';
			$count = count( $snapshot['core_unknown'] );
			$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Additional/suspicious files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __('The following files are not part of the original WordPress package and may have been uploaded by someone else:', 'ninjascanner');
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-2" onChange="nscan_file_info(this.value,2);" style="width:100%;" size="'. select_size($count) .'">';
			foreach( $snapshot['core_unknown'] as $file => $null ) {
				$relative = str_replace( ABSPATH, '', $file );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(2, 'view', null, null, 'quarantine');
			$report['body'] .= '
				</td>
			</tr>';
		}
		// Unknown and suspicious files (Important level):
		if (! empty( $snapshot['core_unknown_root'] ) ) {
			$report['body'] .= '
			<tr>
				<td class="r1"><p>'. __('Unknown files', 'ninjascanner') .'</p></td>
				<td class="r2">';
			$count = count( $snapshot['core_unknown_root'] );
			$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Unknown files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __('The following files are not part of the original WordPress package and may have been uploaded by someone else:', 'ninjascanner');
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-16" onChange="nscan_file_info(this.value,16);" style="width:100%;" size="'. select_size($count) .'">';
			foreach( $snapshot['core_unknown_root'] as $file => $null ) {
				$relative = str_replace( ABSPATH, '', $file );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(16, 'view', null, null, 'quarantine');
			$report['body'] .= '
				</td>
			</tr>';
		}
	}
	$report['body'] .= '
	</table>
	<br />
	<br />';


	// ==================================================================
	// Plugin files integrity
	$total = 0;
	$report['body'] .= '<h3>'. __('Plugin files integrity', 'ninjascanner') .'</h3>
	<table class="widefat">
		<tr>
			<td class="r1"><p>'. __('Plugins', 'ninjascanner') .'</p></td>
			<td class="r2">';
	if (! empty( $snapshot['skip']['scan_pluginsintegrity'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		// Build the lists of modified or added files, and unknown plugins:
		if (! empty( $snapshot['plugins'] ) ) {
			$ok_plugin = array();
			$additional_plugin = array();
			$mismatched_plugin = array();
			foreach( $snapshot['plugins'] as $slug => $arr ) {
				if (! empty( $snapshot['plugins'][$slug] ) ) {
					foreach( $arr as $k => $v ) {
						if ( $v == 1 ) {
							$mismatched_plugin[] = "$slug/$k";
						} else {
							$additional_plugin[] = "$slug/$k";
						}
					}
				} else {
					// OK plugins:
					$ok_plugin[] = $slug;
				}
			}
			// OK plugin files:
			if (! empty( $ok_plugin ) ) {
				if ( empty( $mismatched_plugin ) && empty( $additional_plugin ) &&
					empty( $snapshot['plugins_unknown'] ) && empty( $snapshot['plugins_not_found'] ) &&
					empty( $snapshot['mu_plugins'] ) ) {
					// Plugins are all OK:
					$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				} else {
					$report['body'] .= '<p>'. __('The following plugin files match the original files:', 'ninjascanner' ) . '</p><p>';
					sort( $ok_plugin );
					foreach( $ok_plugin as $slug ) {
						$report['body'] .= '<span class="nscan-ok">&nbsp;'. htmlspecialchars( $slug ) .'&nbsp;</span>&nbsp;&nbsp; ';
					}
					$report['body'] .= '</p><br />';
				}
			}

			// Modified plugin files:
			if (! empty( $mismatched_plugin ) ) {
				$count = count( $mismatched_plugin );
				$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Modified plugin files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __('The following files do not match the original plugin files and may have been tampered with.', 'ninjascanner') .' '. __('If they are damaged or corrupted, you can easily repair them by selecting a file and clicking on the "Restore file" button', 'ninjascanner' ) .':';
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-3" onChange="nscan_file_info(this.value,3);" style="width:100%;" size="'. select_size($count) .'">';
				foreach( $mismatched_plugin as $k => $v ) {
					$file = WP_PLUGIN_DIR . "/$v";
					$relative = str_replace( ABSPATH , '', $file );
					// Populate <option> list:
					$report['body'] .= nscan_build_file_option( $file, $relative );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_info_table(3, 'view', 'compare', 'restore');
			}

			// Unknown plugins dropped inside the /plugins/ folder:
			if (! empty( $snapshot['plugins_unknown'] ) ) {
				foreach( $snapshot['plugins_unknown'] as $slug => $arr ) {
					$additional_plugin[] = $slug;
				}
			}

			// Additional/suspicious plugin files:
			if (! empty( $additional_plugin ) ) {
				$count = count( $additional_plugin );
				$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Additional/suspicious files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __('The following files are not part of any plugin package and may have been uploaded by someone else:', 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-4" onChange="nscan_file_info(this.value,4);" style="width:100%;" size="'. select_size($count) .'">';
				foreach( $additional_plugin as $k => $v ) {
					$file = WP_PLUGIN_DIR . "/$v";
					$relative = str_replace( ABSPATH , '', $file );
					// Populate <option> list:
					$report['body'] .= nscan_build_file_option( $file, $relative );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_info_table(4, 'view', null, null, 'quarantine');
			}
		}

		if (! empty( $snapshot['plugins_not_found'] ) ) {
			// Warn about premium or unknown plugins that couldn't be verified
			// because they aren't available in the wordpress.org repo:
			$report['body'] .= '<p>'. sprintf( __('The following plugin packages could not be compared to the original ones for some reason (more information may be available in <a href="%s">the scanner log</a>). If they are premium plugins, <a href="%s">consult our blog</a> to learn how you can include them in the file integrity checker. Make sure those plugins were not tampered with or installed by someone else:', 'ninjascanner'), '?page=NinjaScanner&nscantab=log', NSCAN_LINK_INTEGRITY_CHECK ) .'</p>';
			ksort( $snapshot['plugins_not_found'] );
			foreach( $snapshot['plugins_not_found'] as $slug => $version ) {
				$report['body'] .= '<span class="nscan-notice">&nbsp;'. htmlspecialchars( $slug .' '. $version ) .'&nbsp;</span>&nbsp;&nbsp; ';
			}
			$report['body'] .= '<br />&nbsp;';
		}

		if (! empty( $snapshot['mu_plugins'] ) ) {
			// Warn about MU plugins:
			$report['body'] .= '<p>'. sprintf( __('The following files are <strong><a href="%s">Must-Use plugins</a></strong> and could not be compared to the original ones. Make sure those plugins were not tampered with or installed by someone else:', 'ninjascanner'), 'https://codex.wordpress.org/Must_Use_Plugins' ) .'</p>';
			foreach( $snapshot['mu_plugins'] as $slug => $version ) {
				$file = str_replace( ABSPATH, '', WP_CONTENT_DIR ."/mu-plugins/$slug" );
				$report['body'] .= '<span class="nscan-warning">&nbsp;'. htmlspecialchars( $file ) .'&nbsp;</span>&nbsp;&nbsp; ';
			}
			$report['body'] .= '<br />&nbsp;';
		}

		if (! empty( $snapshot['plugins_dropins'] ) ) {
			// Warn about drop-ins plugins:
			$count = count( $snapshot['plugins_dropins'] );
			$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Drop-Ins plugins: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __('The following files are <strong>Drop-Ins plugins</strong> (plugins that can be used to replace some core functionality of WordPress) and could not be compared to the original ones. Make sure those plugins were not tampered with or installed by someone else:', 'ninjascanner');
			$report['body'] .= '
				<p>'. $text .'</p>
				<p>'. $mods .'</p>
				<select multiple id="select-report-14" onChange="nscan_file_info(this.value,14);" style="width:100%;" size="'. select_size($count) .'">';
			foreach( $snapshot['plugins_dropins'] as $k => $v ) {
				$file = WP_CONTENT_DIR . "/$k";
				$relative = str_replace( ABSPATH , '', $file );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(14, 'view', null, null, 'quarantine');
		}

		// No plugin found!
		if ( empty( $snapshot['plugins'] ) && empty( $snapshot['plugins_not_found'] ) &&
		empty( $snapshot['plugins_unknown'] ) && empty( $snapshot['plugins_dropins'] ) ) {

			$report['body'] .= '<span class="nscan-warning">&nbsp;'. __('No plugins were found!', 'ninjascanner') .'&nbsp;</span>';
		}
	}
	$report['body'] .= '
			</td>
		</tr>
	</table>
	<br />
	<br />';


	// ==================================================================
	// Theme files integrity
	$total = 0;
	$report['body'] .= '<h3>'. __('Theme files integrity', 'ninjascanner') .'</h3>
	<table class="widefat">
		<tr>
			<td class="r1"><p>'. __('Themes', 'ninjascanner') .'</p></td>
			<td class="r2">';
	if (! empty( $snapshot['skip']['scan_themeseintegrity'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		// Build the lists of modified or added theme files:
		if (! empty( $snapshot['themes'] ) ) {
			$ok_theme = array();
			$additional_theme = array();
			$mismatched_theme = array();
			foreach( $snapshot['themes'] as $slug => $arr ) {
				if (! empty( $snapshot['themes'][$slug] ) ) {
					foreach( $arr as $k => $v ) {

						if ( $v == 1 ) {
							$mismatched_theme[] = "$slug/$k";
						} else {
							$additional_theme[] = "$slug/$k";
						}
					}
				} else {
					// OK themes:
					$ok_theme[] = $slug;
				}
			}
			// OK theme files:
			if (! empty( $ok_theme ) ) {
				if ( empty( $mismatched_theme ) && empty( $additional_theme ) && empty( $snapshot['themes_not_found'] ) ) {
					// Themes are all OK:
					$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				} else {
					$report['body'] .= '<p>'. __('The following theme files match the original files:', 'ninjascanner' ) . '</p><p>';
					sort( $ok_theme );
					foreach( $ok_theme as $slug ) {
						$report['body'] .= '<span class="nscan-ok">&nbsp;'. htmlspecialchars( $slug ) .'&nbsp;</span>&nbsp;&nbsp; ';
					}
					$report['body'] .= '</p><br />';
				}

			}

			// Modified theme files:
			if (! empty( $mismatched_theme ) ) {
				$count = count( $mismatched_theme );
				$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Modified theme files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __('The following files do not match the original theme files and may have been tampered with.', 'ninjascanner') .' '. __('If they are damaged or corrupted, you can easily repair them by selecting a file and clicking on the "Restore file" button', 'ninjascanner' ) .':';
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-5" onChange="nscan_file_info(this.value,5);" style="width:100%;" size="'. select_size($count) .'">';
				foreach( $mismatched_theme as $k => $v ) {
					$file = WP_CONTENT_DIR . "/themes/$v";
					$relative = str_replace( ABSPATH , '', $file );
					// Populate <option> list:
					$report['body'] .= nscan_build_file_option( $file, $relative );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_info_table(5, 'view', 'compare', 'restore');
			}
			// Additional/suspicious theme files:
			if (! empty( $additional_theme ) ) {
				$count = count( $additional_theme );
				$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Additional/suspicious files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __('The following files are not part of any theme package and may have been uploaded by someone else:', 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-6" onChange="nscan_file_info(this.value,6);" style="width:100%;" size="'. select_size($count) .'">';
				foreach( $additional_theme as $k => $v ) {
					$file = WP_CONTENT_DIR . "/themes/$v";
					$relative = str_replace( ABSPATH , '', $file );
					// Populate <option> list:
					$report['body'] .= nscan_build_file_option( $file, $relative );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_info_table(6, 'view', null, null, 'quarantine');
			}
		}

		if (! empty( $snapshot['themes_not_found'] ) ) {
			// Warn about premium or unknown themes that couldn't be verified
			// because they aren't available in the wordpress.org repo:
			$report['body'] .= '<p>'. sprintf( __('The following theme packages could not be compared to the original ones for some reason (more information may be available in <a href="%s">the scanner log</a>). If they are premium themes, <a href="%s">consult our blog</a> to learn how you can include them in the file integrity checker. Make sure those themes were not tampered with or installed by someone else:', 'ninjascanner'), '?page=NinjaScanner&nscantab=log', NSCAN_LINK_INTEGRITY_CHECK ) .'</p>';
			ksort( $snapshot['themes_not_found'] );
			foreach( $snapshot['themes_not_found'] as $slug => $version ) {
				$report['body'] .= '<span class="nscan-notice">&nbsp;'. htmlspecialchars( $slug .' '. $version ) .'&nbsp;</span>&nbsp;&nbsp; ';
			}
			$report['body'] .= '<br />&nbsp;';
		}

		// No themes found (highly unlikely):
		if( empty( $snapshot['themes_not_found'] ) && empty( $snapshot['themes'] ) ) {
			$report['body'] .= '<span class="nscan-warning">&nbsp;'. __('No themes were found!', 'ninjascanner') .'&nbsp;</span>';
		}
	}
	$report['body'] .= '
			</td>
		</tr>
	</table>
	<br />
	<br />';


	// ==================================================================
	// Files & folders
	$report['body'] .= '<h3>'. __('Files and folders', 'ninjascanner') .'</h3>
	<table class="widefat">';

	// Hidden PHP scripts:
	$report['body'] .= '
		<tr>
			<td class="r1"><p>'. __('Hidden scripts', 'ninjascanner') .'</p></td>
			<td class="r2">';
	if (! empty( $snapshot['skip']['core_hidden'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		if (! empty( $snapshot['core_hidden'] ) ) {
			$count = count( $snapshot['core_hidden'] );
			$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Hidden scripts: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __('The following files are hidden PHP scripts:', 'ninjascanner');
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-7" onChange="nscan_file_info(this.value,7);" style="width:100%" size="'. select_size($count) .'">';
			foreach( $snapshot['core_hidden'] as $file => $null ) {
				$relative = str_replace( ABSPATH, '', $file );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(7, 'view', null, null, 'quarantine');
		} else {
			$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
		}
	}
	$report['body'] .= '
			</td>
		</tr>';


	// ==================================================================
	// Binary files
	$report['body'] .= '
		<tr>
			<td class="r1"><p>'. __('Executable files', 'ninjascanner') .'</p></td>
			<td class="r2">';
	if (! empty( $snapshot['skip']['core_binary'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		if (! empty( $snapshot['core_binary'] ) ) {

			$count = count( $snapshot['core_binary'] );
			$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Executable files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __('The following files are executable files:', 'ninjascanner');
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-8" onChange="nscan_file_info(this.value,8);" style="width:100%" size="'. select_size($count) .'">';
			foreach( $snapshot['core_binary'] as $file => $null ) {
				$relative = str_replace( ABSPATH, '', $file );
				$header = '';
				if ( file_exists( $file ) ) {
					$fh = fopen( $file, 'r' );
					if ( ( $data = fread( $fh, 4 ) ) !== false ) {
						if ( preg_match('`^\x7F\x45\x4C\x46`', $data ) ) {
							$header = 'ELF executable format';
						} elseif( preg_match('`^\x4D\x5A`', $data ) ) {
							$header = 'Microsoft MZ executable format';
						}
					}
					fclose( $fh );
					if ( empty( $header ) ) {
						$header = 'Unknown format';
					}
				}
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative, 8, $header );


			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(8, null, null, null, 'quarantine');
		} else {
			$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
		}
	}
	$report['body'] .= '
			</td>
		</tr>';

	// ==================================================================
	// Symlink files
	$report['body'] .= '
		<tr>
			<td class="r1"><p>'. __('Symbolic links', 'ninjascanner') .'</p></td>
			<td class="r2">';
	if (! empty( $snapshot['skip']['core_symlink'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		if (! empty( $snapshot['core_symlink'] ) ) {

			$count = count( $snapshot['core_symlink'] );
			$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Symbolink links: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __('The following files or folders are symbolic links:', 'ninjascanner');
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-9" onChange="nscan_file_info(this.value,9);" style="width:100%" size="'. select_size($count) .'">';
			foreach( $snapshot['core_symlink'] as $file => $null ) {
				$relative = str_replace( ABSPATH, '', $file );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative, 9 );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(9);
		} else {
			$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
		}
	}
	$report['body'] .= '
				</td>
			</tr>';

	// ==================================================================
	// Unreadable files/folders
	$report['body'] .= '
		<tr>
			<td class="r1"><p>'. __('Unreadable files/folders', 'ninjascanner') .'</p></td>
			<td class="r2">';
	if (! empty( $snapshot['skip']['core_unreadable'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		if (! empty( $snapshot['core_unreadable'] ) ) {

			$count = count( $snapshot['core_unreadable'] );
			$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Unreadable files/folders: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __("The following files or folders are unreadable, NinjaScanner couldn't scan them:", 'ninjascanner');
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-10" onChange="nscan_file_info(this.value,10);" style="width:100%" size="'. select_size($count) .'">';
			foreach( $snapshot['core_unreadable'] as $file => $null ) {
				$relative = str_replace( ABSPATH, '', $file );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(10);
		} else {
			$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
		}
	}
	$report['body'] .= '
				</td>
			</tr>
		</table>
		<br />
		<br />';

	// ==================================================================
	// Google Safe Browsing
	$report['body'] .= '<h3>'. __('Google Safe Browsing', 'ninjascanner') .'</h3>
		<table class="widefat">
			<tr>
				<td class="r1"><p>'. __('Malware and phishing diagnostic', 'ninjascanner') .'</p></td>
				<td class="r2">';
	// Error?
	if (! empty( $snapshot['step_error'][8] ) ) {
		$report['body'] .= '<p><span class="nscan-critical">&nbsp;'. sprintf(
			__('Warning, a critical error occurred. This test was cancelled: %s', 'ninjascanner'),
			$snapshot['step_error'][8]
		) ."&nbsp;</span></p>";

	} elseif (! empty( $snapshot['skip']['scan_gsb'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		// Infected URLs:
		if (! empty( $snapshot['scan_gsb'] ) ) {
			$count = count(  $snapshot['scan_gsb'] );
			$report['body'] .= '<span class="nscan-critical">&nbsp;'. sprintf( __('Total websites listed on Google Safe Browsing blacklist: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$report['body'] .= '<ul>';
			foreach( $snapshot['scan_gsb'] as $site => $v ) {
				$report['body'] .= '<li><a target="_blank" title="'. __('Click on the link to access the diagnostic page.', 'ninjascanner') .'" href="http://safebrowsing.clients.google.com/safebrowsing/diagnostic?site='. urlencode( $site ) .'">'. htmlspecialchars( $site ) .'<a/></li>';
			}
			$report['body'] .= '</ul>';

		} else {
			$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
		}
	}
	$report['body'] .= '
		</table>
		<br />
		<br />';

	// ==================================================================
	// Anti-malware
	$report['body'] .= '<h3>'. __('Anti-malware', 'ninjascanner') .'</h3>
		<table class="widefat">
			<tr>
				<td class="r1"><p>'. __('Suspicious files', 'ninjascanner') .'</p></td>
				<td class="r2">';

	// Error?
	if (! empty( $snapshot['step_error'][9] ) ) {
		$report['body'] .= '<p><span class="nscan-critical">&nbsp;'. sprintf(
			__('Warning, a critical error occurred. This test was cancelled: %s', 'ninjascanner'),
			$snapshot['step_error'][9]
		) ."&nbsp;</span></p>";

	} elseif (! empty( $snapshot['skip']['scan_antimalware'] ) ) {
		$report['body'] .= '<p><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></p>';

	} else {
		// Infected files:
		if (! empty( $snapshot['infected_files'] ) ) {
			$count = count( $snapshot['infected_files'] );
			$mods = '<span class="nscan-critical">&nbsp;'. sprintf( __('Suspicious files: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
			$text = __("The following files may be infected with malicious code:", 'ninjascanner');
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-11" onChange="nscan_file_info(this.value,11);" style="width:100%" size="'. select_size($count) .'">';
			foreach( $snapshot['infected_files'] as $file => $v ) {
				$relative = str_replace( ABSPATH, '', $file );
				$v = str_replace( array( '{REX}' , '{HEX}' ), '', $v );
				// Populate <option> list:
				$report['body'] .= nscan_build_file_option( $file, $relative, 11, $v );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(11, 'view', null, null, 'quarantine' );
			$report['body'] .= '
				</td>
			</tr>';
		} else {
			$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
		}
	}
	$report['body'] .= '
		</table>
		<br />
		<br />';


	// ==================================================================
	// File snapshot
	$report['body'] .= '<h3>'. __('File snapshot', 'ninjascanner') .'</h3>
		<table class="widefat">';

	if (! empty( $snapshot['skip']['scan_warnfilechanged'] ) ) {
		$report['body'] .= '
			<tr>
				<td class="r1"><p>'. __('Files', 'ninjascanner') .'</p></td>
				<td class="r2"><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></td>
			</tr>';

	} else {
		if (! empty( $snapshot['snapshot']['mismatched_files'] ) ||
			! empty( $snapshot['snapshot']['added_files'] ) ||
			! empty( $snapshot['snapshot']['deleted_files'] ) ) {


			// Files modified since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('Modified files', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['mismatched_files'] ) ) {
				$count = count( $snapshot['snapshot']['mismatched_files'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Modified files since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following files have been modified since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
						<select multiple id="select-report-12" onChange="nscan_file_info(this.value,12);" style="width:100%" size="'. select_size($count) .'">';
				foreach( $snapshot['snapshot']['mismatched_files'] as $file => $v ) {
					$relative = str_replace( ABSPATH, '', $file );
					// Populate <option> list:
					$report['body'] .= nscan_build_file_option( $file, $relative );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_info_table(12, 'view');
				$report['body'] .= '
					</td>
				</tr>';
			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

			// Files added since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('New files', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['added_files'] ) ) {
				$count = count( $snapshot['snapshot']['added_files'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Added files since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following files have been added since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
						<select multiple id="select-report-13" onChange="nscan_file_info(this.value,13);" style="width:100%" size="'. select_size($count) .'">';
				foreach( $snapshot['snapshot']['added_files'] as $file => $v ) {
					$relative = str_replace( ABSPATH, '', $file );
					// Populate <option> list:
					$report['body'] .= nscan_build_file_option( $file, $relative );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_info_table(13, 'view');
				$report['body'] .= '
					</td>
				</tr>';
			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

			// Files deleted since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('Deleted files', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['deleted_files'] ) ) {
				$count = count( $snapshot['snapshot']['deleted_files'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Deleted files since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following files have been deleted since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
						<select multiple onChange="nscan_file_deleted();" style="width:100%" size="'. select_size($count) .'">';
				foreach( $snapshot['snapshot']['deleted_files'] as $file => $v ) {
					$relative = str_replace( ABSPATH, '', $file );
					$report['body'] .= '<option class="deleted" title="'. htmlspecialchars( $file ) .'">'. htmlspecialchars( $relative  ) .'</option>';
				}
				$report['body'] .= '</select><br />&nbsp;
					</td>
				</tr>';
			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}
		} else {
			if (! file_exists( NSCAN_OLD_SNAPSHOT ) ) {
				$report['body'] .= '
					<tr>
						<td class="r1"><p>'. __('Files', 'ninjascanner') .'</p></td>
						<td class="r2"><p>'. __('Skipping snapshots comparison, no older file snapshot found', 'ninjascanner') .'.</p></td>
					</tr>';
			} else {
				$report['body'] .= '
					<tr>
						<td class="r1"><p>'. __('Files', 'ninjascanner') .'</p></td>
						<td class="r2"><p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p></td>
					</tr>';
			}
		}
	}
	$report['body'] .= '
		</table>
		<br />
		<br />';


	// ==================================================================
	// Database snapshot


	// Posts:
	$report['body'] .= '<h3>'. __('Database snapshot (posts)', 'ninjascanner') .'</h3>
		<table class="widefat">';

	if (! empty( $snapshot['skip']['scan_warndbchanged'] ) ) {
		$report['body'] .= '
			<tr>
				<td class="r1"><p>'. __('Posts', 'ninjascanner') .'</p></td>
				<td class="r2"><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></td>
			</tr>';

	} else {
		if (! empty( $snapshot['snapshot']['mismatched_posts'] ) ||
			! empty( $snapshot['snapshot']['added_posts'] ) ||
			! empty( $snapshot['snapshot']['deleted_posts'] ) ) {

			// Posts modified since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('Modified posts', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['mismatched_posts'] ) ) {
				$count = count( $snapshot['snapshot']['mismatched_posts'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Modified posts since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following posts have been modified since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-17" onChange="nscan_post_info(this.value,17);" style="width:100%" size="'. select_size($count) .'">';

				foreach( $snapshot['snapshot']['mismatched_posts'] as $id => $val ) {
					// Populate list:
					$report['body'] .= nscan_build_post_option( 17, $id, $val, 'post' );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_post_info_table(17, 'view');
				$report['body'] .= '
					</td>
				</tr>';

			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

			// Posts added since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('New posts', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['added_posts'] ) ) {
				$count = count( $snapshot['snapshot']['added_posts'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Added posts since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following posts have been added since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-18" onChange="nscan_post_info(this.value,18);" style="width:100%" size="'. select_size($count) .'">';

				foreach( $snapshot['snapshot']['added_posts'] as $id => $val ) {
					// Populate list:
					$report['body'] .= nscan_build_post_option( 18, $id, $val, 'post' );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_post_info_table(18, 'view');
				$report['body'] .= '
					</td>
				</tr>';

			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

			// Posts deleted since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('Deleted posts', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['deleted_posts'] ) ) {
				$count = count( $snapshot['snapshot']['deleted_posts'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Deleted posts since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following posts have been deleted since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-19" onChange="nscan_post_deleted();" style="width:100%" size="'. select_size($count) .'">';

				foreach( $snapshot['snapshot']['deleted_posts'] as $id => $val ) {
					// Populate list:
					$report['body'] .= nscan_build_post_option( 19, $id, $val, 'post' );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_post_info_table(19, null );
				$report['body'] .= '
					</td>
				</tr>';

			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

		} else {
			if (! file_exists( NSCAN_OLD_SNAPSHOT ) ) {
				$report['body'] .= '
					<tr>
						<td class="r1"><p>'. __('Posts', 'ninjascanner') .'</p></td>
						<td class="r2"><p>'. __('Skipping snapshots comparison, no older database snapshot found', 'ninjascanner') .'.</p></td>
					</tr>';
			} else {
				$report['body'] .= '
					<tr>
						<td class="r1"><p>'. __('Posts', 'ninjascanner') .'</p></td>
						<td class="r2"><p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p></td>
					</tr>';
			}
		}
	}

	$report['body'] .= '
		</table>
		<br />
		<br />';


	// Pages:
	$report['body'] .= '<h3>'. __('Database snapshot (pages)', 'ninjascanner') .'</h3>
		<table class="widefat">';

	if (! empty( $snapshot['skip']['scan_warndbchanged'] ) ) {
		$report['body'] .= '
			<tr>
				<td class="r1"><p>'. __('Pages', 'ninjascanner') .'</p></td>
				<td class="r2"><span class="nscan-notice">&nbsp;'. __('This test was skipped', 'ninjascanner') .'</span></td>
			</tr>';

	} else {
		if (! empty( $snapshot['snapshot']['mismatched_pages'] ) ||
			! empty( $snapshot['snapshot']['added_pages'] ) ||
			! empty( $snapshot['snapshot']['deleted_pages'] ) ) {

			// Pages modified since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('Modified pages', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['mismatched_pages'] ) ) {
				$count = count( $snapshot['snapshot']['mismatched_pages'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Modified pages since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following pages have been modified since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-20" onChange="nscan_post_info(this.value,20);" style="width:100%" size="'. select_size($count) .'">';

				foreach( $snapshot['snapshot']['mismatched_pages'] as $id => $val ) {
					// Populate list:
					$report['body'] .= nscan_build_post_option( 20, $id, $val, 'page' );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_post_info_table(20, 'view');
				$report['body'] .= '
					</td>
				</tr>';

			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

			// Pages added since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('New pages', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['added_pages'] ) ) {
				$count = count( $snapshot['snapshot']['added_pages'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Added pages since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following pages have been added since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-21" onChange="nscan_post_info(this.value,21);" style="width:100%" size="'. select_size($count) .'">';

				foreach( $snapshot['snapshot']['added_pages'] as $id => $val ) {
					// Populate list:
					$report['body'] .= nscan_build_post_option( 21, $id, $val, 'page' );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_post_info_table(21, 'view');
				$report['body'] .= '
					</td>
				</tr>';

			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

			// Pages deleted since last scan:
			$report['body'] .= '
				<tr>
					<td class="r1"><p>'. __('Deleted pages', 'ninjascanner') .'</p></td>
					<td class="r2">';
			if (! empty( $snapshot['snapshot']['deleted_pages'] ) ) {
				$count = count( $snapshot['snapshot']['deleted_pages'] );
				$mods = '<span class="nscan-warning">&nbsp;'. sprintf( __('Deleted pages since last scan: %s', 'ninjascanner'), $count ) .'&nbsp;</span>';
				$text = __("The following pages have been deleted since last scan:", 'ninjascanner');
				$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-22" onChange="nscan_post_deleted();" style="width:100%" size="'. select_size($count) .'">';

				foreach( $snapshot['snapshot']['deleted_pages'] as $id => $val ) {
					// Populate list:
					$report['body'] .= nscan_build_post_option( 22, $id, $val, 'page' );
				}
				$report['body'] .= '</select>';
				// Display file info table:
				$report['body'] .= nscan_build_post_info_table(22, null );
				$report['body'] .= '
					</td>
				</tr>';

			} else {
				$report['body'] .= '<p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p>';
				$report['body'] .= '
					</td>
				</tr>';
			}

		} else {
			if (! file_exists( NSCAN_OLD_SNAPSHOT ) ) {
				$report['body'] .= '
					<tr>
						<td class="r1"><p>'. __('Pages', 'ninjascanner') .'</p></td>
						<td class="r2"><p>'. __('Skipping snapshots comparison, no older database snapshot found', 'ninjascanner') .'.</p></td>
					</tr>';
			} else {
				$report['body'] .= '
					<tr>
						<td class="r1"><p>'. __('Pages', 'ninjascanner') .'</p></td>
						<td class="r2"><p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p></td>
					</tr>';
			}
		}
	}

	$report['body'] .= '
		</table>
		<br />
		<br />';


	// ==================================================================
	// Various tests

	if (! defined('NFW_STATUS') && PATH_SEPARATOR != ';' ) {
		// Ignore it if we are running on MS Windows server:
		$snapshot['various']['waf'] = 0;
	}

	if (! empty( $snapshot['various'] ) ) {

		$report['body'] .= '<h3>'. __('Various', 'ninjascanner') .'</h3>
		<table class="widefat">';

		if (! empty( $snapshot['various']['ssh_key'] ) ) {
			$count = count( $snapshot['various']['ssh_key'] );
			$report['body'] .= '<tr><td class="r1"><p>'. __('SSH key', 'ninjascanner') .'</p></td>
				<td class="r2">';
			$mods = '<span class="nscan-warning">&nbsp;'. __('SSH key', 'ninjascanner') .'&nbsp;</span>';
			$text = sprintf(
				_n(
					'%s SSH key was found and allows a user to connect to your site over SSH. Make sure that file was not uploaded by someone else:',
					'%s SSH keys were found and allow a user to connect to your site over SSH. Make sure those files were not uploaded by someone else:',
					$count,
					'ninjascanner'
				),
				$count
			);
			$report['body'] .= '
					<p>'. $text .'</p>
					<p>'. $mods .'</p>
					<select multiple id="select-report-15" onChange="nscan_file_info(this.value,15);" style="width:100%" size="'. select_size($count) .'">';
			foreach( $snapshot['various']['ssh_key'] as $k => $v ) {
				// Populate list:
				$report['body'] .= nscan_build_file_option( $k, $k );
			}
			$report['body'] .= '</select>';
			// Display file info table:
			$report['body'] .= nscan_build_info_table(15, 'view', null, null, 'quarantine');
			$report['body'] .= '
				</td>
			</tr>';
		}

		if (! empty( $snapshot['various']['membership'] ) ) {
			$report['body'] .= '<tr><td class="r1"><p>'. __('Membership', 'ninjascanner') .'</p></td>
				<td class="r2">';
			if ( $snapshot['various']['membership'] == 1 ) {
				$report['body'] .= '<span class="nscan-warning">&nbsp;'. __('Although user registration is disabled, the "New User Default Role" option is set to "administrator".', 'ninjascanner') .'&nbsp;</span>&nbsp;&nbsp; ';
			} else {
				$report['body'] .= '<span class="nscan-critical">&nbsp;'. __('User registration is enabled and the "New User Default Role" option is set to "administrator"!', 'ninjascanner') .'&nbsp;</span>&nbsp;&nbsp; ';
			}
			$report['body'] .= '
				</td>
			</tr>';
		}

		if (! empty( $snapshot['various']['user_roles'] ) ) {
			$report['body'] .= '<tr><td class="r1"><p>'. __('User roles:', 'ninjascanner') .'</p></td>
				<td class="r2">';
			$report['body'] .= '<p>'. sprintf( __('NinjaScanner has detected that the following user roles have been given <a href="%s">capabilities</a> that, by default, are only assigned to an administrator:', 'ninjascanner'), 'https://codex.wordpress.org/Roles_and_Capabilities#Capability_vs._Role_Table' ) .'</p>';

			foreach( $snapshot['various']['user_roles'] as $user => $cap   ) {
				$report['body'] .= "<p>Role: {$user} - Capabilities: ";
				foreach( $cap as $k => $v ) {
					$report['body'] .= '<span class="nscan-warning">&nbsp;'. $v .'&nbsp;</span>&nbsp;&nbsp; ';
				}
				$report['body'] .= "</p>";
			}
			$report['body'] .= '
				</td>
			</tr>';
		}

		if ( isset( $snapshot['various']['waf'] ) ) {
			$report['body'] .= '<tr><td class="r1"><p>'. __('Firewall', 'ninjascanner') .'</p></td>
				<td class="r2">
					<p><span class="nscan-notice">&nbsp;'. __('No firewall detected', 'ninjascanner') .'&nbsp;</span></p>
					<p>'. sprintf( __('Consider installing a Web Application Firewall such as <a href="%s">NinjaFirewall (WP Edition)</a> to make sure your site is well protected against web attacks.', 'ninjascanner'), 'https://wordpress.org/plugins/ninjafirewall/' ) .'</p>
				</td>
			</tr>';
		}

	} else {
		$report['body'] .= '<h3>'. __('Various', 'ninjascanner') .'</h3>
		<table class="widefat">
			<tr>
				<td class="r1"><p>&nbsp;</p></td>
				<td class="r2"><p><span class="nscan-ok">&nbsp;'. __('No problem detected', 'ninjascanner') .'&nbsp;</span></p></td>
			</tr>';
	}
	$report['body'] .= '
	</table>
	<br />
	<br />';

	return $report;

}

// =====================================================================
// Set the height of the select box depending on the number
// of elements to display.

function select_size( $count ) {

	//Min/max allowed size:
	$min_size = 2;
	$max_size = 10;

	if ( $count < $min_size ) {
		return $min_size;
	} elseif ( $count > $max_size ) {
		return $max_size;
	}
	return $count;
}

// =====================================================================
// Build the list of file options fields inside the <select> tag.

function nscan_build_file_option( $file, $relative, $id = 0, $extra = 0 ) {

	// Make sure the file wasn't deleted since last scan:
	if (! file_exists( $file ) ) {

		$missing =  __('(deleted)', 'ninjascanner' );

		// Check if it was quarantined:
		if ( file_exists( NSCAN_QUARANTINE .'/quarantine.php' ) ) {
			if ( nscan_is_json_encoded( NSCAN_QUARANTINE .'/quarantine.php' ) === true ) {
				$quarantined_files = json_decode( file_get_contents( NSCAN_QUARANTINE .'/quarantine.php' ), true );
			} else {
				$quarantined_files = unserialize( file_get_contents( NSCAN_QUARANTINE .'/quarantine.php' ) );
			}
			if (! empty( $quarantined_files[$file] ) ) {
				$missing = __('(quarantined)', 'ninjascanner');
			}
		}
		return '<option class="deleted" value="deleted">'. htmlspecialchars( $relative ) .' '. $missing .'</option>';

	} else {
		# Get information about the file:
		$stat = stat( $file );
		$tmp = base64_encode( $file ) .
			"::{$stat['uid']}::{$stat['gid']}::";

		/// If this is a symlink, we collect the link target, not its size:
		if ( $id == 9 ) {
			$tmp .= readlink( $file ) ."::";
		} else {
			$tmp .= number_format_i18n( $stat['size'] ) ." ". __('bytes', 'ninjascanner') ."::";
		}
		$tmp .= date_i18n( 'M d, Y, H:i:s O', $stat['mtime']) ."::" .
			date_i18n( 'M d, Y, H:i:s O', $stat['ctime']) ."::";

		// Convert permissions to a human readable format:
		$p = sprintf ("%04o", $stat['mode'] & 0777);
		$flags = array( '---', '--x', '-w-', '-wx', 'r--', 'r-x', 'rw-', 'rwx' );
		if ( is_dir( $file ) ) {
			$permissions = 'd';
		} else {
			$permissions = '-';
		}
		$permissions .= $flags[$p[1]] . $flags[$p[2]] . $flags[$p[3]];
		$tmp .= "{$p} {$permissions}";

		if ( $id == 11 || $id == 8 ) {
			// -Antimalware: Add the reason why the file was tagged as suspicious.
			// -Binary file: Add its format (PE/NE/MZ/ELF).
			$tmp .= "::{$extra}";
		}

		return '<option value="'. htmlspecialchars( $tmp ) .'" title="'.
			htmlspecialchars( $file ) .'">'. htmlspecialchars( $relative ) .'</option>';
	}
}

// =====================================================================
// Display the file info table below the list of files.

function nscan_build_info_table( $id, $view = 0, $compare = 0, $restore = 0, $quarantine = 0 ) {

	$nonce = wp_create_nonce( 'nscan_file_op' );

	$buffer =
	'<center>
		<span class="description">'. __('Click a file to get more info about it.', 'ninjascanner') .'</span>
		<div id="table-report-'. $id .'" style="display:none">

			<table style="width:100%;background-color:#F7F7F7;border:solid 1px #DFDFDF;" cellspacing="4">';
				// Show the reason why the file was tagged a suspicious
				// if this is the antimalware results:
				if ( $id == 11 ) {
					$buffer .= '
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">'. __('Detection', 'ninjascanner') .'</th>
					<td style="padding:0;width:60%;font-weight:bold" id="table-report-'. $id .'-lmd"></td>
					<td style="padding:0;width:20%">&nbsp;</td>
				</tr>';
				}

				// If it is a binary, display its format:
				if ( $id == 8 ) {
					$buffer .= '
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">'. __('File type', 'ninjascanner') .'</th>
					<td style="padding:0;width:60%;font-weight:bold" id="table-report-'. $id .'-bin"></td>
					<td style="padding:0;width:20%">&nbsp;</td>
				</tr>';
				}

				$buffer .= '
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">';
					// Show target instead of size if this is a symlink:
					if ( $id == 9 ) {
						$buffer .= __('Target', 'ninjascanner'). '</th>
						<td style="padding:0;width:60%"><code id="table-report-'. $id .'-size"></code></td>';
					} else {
						$buffer .=__('Size', 'ninjascanner'). '</th>
						<td style="padding:0;width:60%" id="table-report-'. $id .'-size"></td>';
					}
					$buffer .= '
					<td style="padding:0;width:20%;text-align:center">';
					// View file:
					if (! empty( $view ) ) {
						$buffer .= '<input type="button" onClick="nscan_op(document.getElementById(\'table-report-'.$id.'-fname\').value, \'view\', \''. $nonce .'\', \''. $id .'\', document.getElementById(\'table-report-'.$id.'-value\').value);" class="button-secondary" style="width:100%;" title="' . __('View the selected file', 'ninjascanner') .'" value="' . __('View file', 'ninjascanner') .'" />';

					} else {
						$buffer .= '<input type="button" class="button-secondary" style="width:100%;" disabled="disabled" value="' . __('View file', 'ninjascanner') .'" />';
					}
					$buffer .= '
					</td>
				</tr>
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">'. __('Access', 'ninjascanner') .'</th>
					<td style="padding:0;width:60%" id="table-report-'. $id .'-mode"></td>
					<td style="padding:0;width:20%;text-align:center">';
					if (! empty( $compare ) ) {
						$buffer .= '<input type="button" onClick="nscan_op(document.getElementById(\'table-report-'.$id.'-fname\').value, \'compare\', \''. $nonce .'\', \''. $id .'\', document.getElementById(\'table-report-'.$id.'-value\').value);" class="button-secondary" style="width:100%;" title="' . __('Compare the selected file to the original one', 'ninjascanner') .'" value="' . __('View changes', 'ninjascanner') .'" />';
					} else {
						$buffer .= '<input type="button" class="button-secondary" style="width:100%;" disabled="disabled" value="' . __('View changes', 'ninjascanner') .'" />';
					}
					$buffer .= '
					</td>
				</tr>
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">'. __('Uid / Gid', 'ninjascanner') .'</th>
					<td style="padding:0;width:60%" id="table-report-'. $id .'-uidgid"></td>
					<td style="padding:0;width:20%;text-align:center">';
					if (! empty( $restore ) ) {
						$buffer .= '<input id="restore-btn-'. $id .'" type="button" onClick="nscan_op(document.getElementById(\'table-report-'.$id.'-fname\').value, \'restore\', \''. $nonce .'\', \''. $id .'\', document.getElementById(\'table-report-'.$id.'-value\').value);" class="button-secondary" style="width:100%;" title="' . __('Restore the original file', 'ninjascanner') .'" value="' . __('Restore file', 'ninjascanner') .'" /><img id="restore-img-'. $id .'" style="vertical-align:middle;display:none" src="'. plugins_url() .'/ninjascanner/static/progress.gif" />';
					} else {
						$buffer .= '<input type="button" class="button-secondary" style="width:100%;" disabled="disabled" value="' . __('Restore file', 'ninjascanner') .'" />';
					}

					$buffer .= '
					</td>
				</tr>
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">'. __('Modify', 'ninjascanner') .'</th>
					<td style="padding:0;width:60%" id="table-report-'. $id .'-mtime"></td>
					<td style="padding:0;width:20%;text-align:center">';
					if (! empty( $quarantine ) ) {
						$buffer .= '<input id="quarantine-btn-'. $id .'" type="button" onClick="nscan_op(document.getElementById(\'table-report-'.$id.'-fname\').value, \'quarantine\', \''. $nonce .'\', \''. $id .'\', document.getElementById(\'table-report-'.$id.'-value\').value);" class="button-secondary" style="width:100%;" title="' . __('Quarantine the file', 'ninjascanner') .'" value="' . __('Quarantine file', 'ninjascanner') .'" /><img id="quarantine-img-'. $id .'" style="vertical-align:middle;display:none" src="'. plugins_url() .'/ninjascanner/static/progress.gif" />';
					} else {
						$buffer .= '<input type="button" class="button-secondary" style="width:100%;" disabled="disabled" value="' . __('Quarantine file', 'ninjascanner') .'" />';
					}
					$buffer .= '
					</td>
				</tr>
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">'. __('Change', 'ninjascanner') .'</th>
					<td style="padding:0;width:60%" id="table-report-'. $id .'-ctime"></td>
					<td style="padding:0;width:20%;text-align:center">
					<input type="button" onClick="nscan_close_info('. $id .')" class="button-secondary" style="width:100%;" title="' . __('Close this table', 'ninjascanner') .'" value="' . __('Close', 'ninjascanner') .'" /></td>
				</tr>
			</table>
			<input type="hidden" value="" id="table-report-'. $id .'-fname" />
			<input type="hidden" value="" id="table-report-'. $id .'-value" />
		</div>
	</center>
	<br />&nbsp;';

	return $buffer;

}

// =====================================================================
// Build the list of posts/pages options fields inside the <select> tag.

function nscan_build_post_option( $id, $post_id, $permalink, $post_type ) {

	// We only care about published posts/pages:
	if ( get_post_status( (int) $post_id ) != 'publish' ) {
		return '<option class="deleted" value="deleted" title="ID '.
			(int) $post_id .'">'. htmlspecialchars( $permalink ) .' '.
			__('(deleted or unpublished)', 'ninjascanner' ) .'</option>';

	} else {

		if ( $post_type != 'post' ) {
			$post_type = 'page';
		}
		$tmp = (int) $post_id ."::{$post_type}";
		return '<option value="'. $tmp .'" title="'.
			htmlspecialchars( $permalink ) .'">'. htmlspecialchars( $permalink ) .
			'</option>';
	}
}

// =====================================================================
// Display the pages/posts info table below the list of items.

function nscan_build_post_info_table( $id, $view = 0 ) {

	$buffer =
	'<center>
		<span class="description">'. __('Click a post to get more info about it.', 'ninjascanner') .'</span>
		<div id="table-report-'. $id .'" style="display:none">
			<table style="width:100%;background-color:#F7F7F7;border:solid 1px #DFDFDF;" cellspacing="4">
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">';
					$buffer .=__('ID', 'ninjascanner'). '</th>
					<td style="padding:0;width:60%" id="table-report-'. $id .'-id"></td>';
					$buffer .= '
					<td style="padding:0;width:20%;text-align:center">';
					// We can view it only if it wasn't deleted:
					if (! empty( $view ) ) {
						$buffer .= '<input type="button" onClick="nscan_op_post(document.getElementById(\'table-report-'.$id.'-value\').value);" class="button-secondary" style="width:100%;" title="' . __('View the selected post', 'ninjascanner') .'" value="' . __('View post', 'ninjascanner') .'" />';

					} else {
						$buffer .= '<input type="button" class="button-secondary" style="width:100%;" disabled="disabled" value="' . __('View post', 'ninjascanner') .'" />';
					}
					$buffer .= '</td>
				</tr>
				<tr>
					<th style="padding:0;width:20%;font-weight:bold">';
					$buffer .=__('Type', 'ninjascanner'). '</th>
					<td style="padding:0;width:60%" id="table-report-'. $id .'-type"></td>';
					$buffer .= '
					<td style="padding:0;width:20%;text-align:center">';
					$buffer .= '<input type="button" onClick="nscan_close_info('. $id .')" class="button-secondary" style="width:100%;" title="' . __('Close this table', 'ninjascanner') .'" value="' . __('Close', 'ninjascanner') .'" />';
					$buffer .= '</td>
				</tr>
			</table>
			<input type="hidden" value="" id="table-report-'. $id .'-value" />
		</div>
	</center>
	<br />&nbsp;';

	return $buffer;

}

/* ================================================================== */
// EOF
