<?php
/*
 +=====================================================================+
 |     _   _ _        _       ____                                     |
 |    | \ | (_)_ __  (_) __ _/ ___|  ___ __ _ _ __  _ __   ___ _ __    |
 |    |  \| | | '_ \ | |/ _` \___ \ / __/ _` | '_ \| '_ \ / _ \ '__|   |
 |    | |\  | | | | || | (_| |___) | (_| (_| | | | | | | |  __/ |      |
 |    |_| \_|_|_| |_|/ |\__,_|____/ \___\__,_|_| |_|_| |_|\___|_|      |
 |                 |__/                                                |
 |                                                                     |
 | (c) NinTechNet ~ https://nintechnet.com/                            |
 +=====================================================================+ // sa+i18n
*/

if (! defined( 'ABSPATH' ) ) { die( 'Forbidden' ); }

$disable_scan = '';
if (! is_dir( NSCAN_CACHEDIR ) ) {
	$message = sprintf( __('NinjaScanner cannot write to the <code>%s</code> folder.', 'ninjascanner' ) . ' ' .
				__('Is your filesystem read-only?', 'ninjascanner'), htmlspecialchars( NSCAN_CACHEDIR ) );
	$error = 1;
	$disable_scan = ' disabled="disabled"';
}

// Check if a scan is currently running:
$nscan_is_running = 0;
if ( nscan_is_scan_running() !== false ) {
	$nscan_is_running = 1;
}

// Submitted form ?
if (! empty( $_POST ) ) {

	// Check nonce:
	if ( empty( $_POST['nscannonce'] ) || ! wp_verify_nonce($_POST['nscannonce'], 'nscan_summary') ) {
		wp_nonce_ays('nscan_summary');
	}

	// Run scan:
	if (! empty( $_POST['start-scan'] ) ) {
		// Make sure we dont have a scan already running:
		if (! $nscan_is_running ) {
			// Start it:
			file_put_contents( NSCAN_DEBUGLOG, time() .'~~1~~'. __('Scheduling a scanning process', 'ninjascanner') ."\n" );
			if ( nscan_shedule_event() === false ) {
				$message = sprintf(
					__('Error: Cannot start the scan. More details may be available in <a href="%s">the scanner log</a>.', 'ninjascanner' ),
					'?page=NinjaScanner&nscantab=log'
				) .'<br />'.	__('If this error persists, select a different value for the "Settings > Advanced Users Settings > Scanning process > Fork process" option.', 'ninjascanner' );
				$error = 1;
			} else {
				// All done, it's running:
				$nscan_is_running = 1;
			}
		} else {
			// Already running:
			$message = __('A scanning process is running. Please wait or stop it.', 'ninjascanner' );
			$error = 1;
		}

	// Cancel scan:
	} elseif (! empty( $_POST['cancel-scan'] ) ) {

		// Make sure we have a running scan:
		if (! $nscan_is_running ) {
			$message = __('No scanning process is running.', 'ninjascanner' );
			$error = 1;

		} else {
			$nscan_is_running = 0;

			// Cancel it:
			$err_code = nscan_cancel_scan();
			if ( $err_code == 1 ) {
				$message = sprintf( __('Error: cannot find the lock file (%s).', 'ninjascanner' ) .'<br />' .
					__('Is NinjaScanner running?', 'ninjascanner'), '<code>' . htmlspecialchars( NSCAN_LOCKFILE ) .'</code>' );
				$error = 1;
			} elseif ( $err_code == 2 ) {
				$message = sprintf( __('Error: cannot delete the lock file (%s).', 'ninjascanner' ) .'<br />' .
					__('Is your filesystem read-only?', 'ninjascanner'), '<code>' . htmlspecialchars( NSCAN_LOCKFILE ) .'</code>' );
				$error = 1;
			} else {
				// Successfully cancelled:
				$message = __('The scanning process was cancelled.', 'ninjascanner' );
			}
		}
	} elseif (! empty($_POST['view-report'] ) ) {
		$report = array();
		require_once __DIR__ . '/report_html.php';
		$report = html_report();
		if (! empty( $report['error'] ) ) {
			$error = 1;
			$message = $report['error'];
		}
	}
}

$nscan_options = get_option( 'nscan_options' );

// A scan is running, display the notice:
if ( $nscan_is_running ) {
	$message = __('NinjaScanner is running in the background.', 'ninjascanner') . ' ' .
				__('Meanwhile, you can leave this page and keep working on your blog as usual, or even log out of the WordPress dashboard.', 'ninjascanner');
	if (! empty( $nscan_options['admin_email'] ) ) {
		$message .=  ' ' . sprintf( __('A report will be sent to %s as soon as the scan has finished.', 'ninjascanner' ), '<code>'. htmlspecialchars( $nscan_options['admin_email'] ) .'</code>' );
	}
	$nscan_is_running = 1;
	$notice = 1;

// If last scan ended with a fatal error, we display it once to the user:
} elseif ( file_exists( NSCAN_CACHEDIR . '/error.txt' ) ) {
	$message = __('Last scanning process failed and returned the following message:', 'ninjascanner') .'<br />';
	$message .= htmlspecialchars( file_get_contents( NSCAN_CACHEDIR . '/error.txt' ) );
	$error = 1;
	unlink( NSCAN_CACHEDIR . '/error.txt' );
}

if (! empty( $message ) ) {
	if ( isset( $error ) ) {
		echo '<div class="error notice is-dismissible"><p>' . $message . '</p></div>';
	} elseif ( isset( $notice ) ) {
		echo '<div class="notice-info notice is-dismissible"><p>' . $message . '</p></div>';
	} else {
		echo '<div class="updated notice is-dismissible"><p>' . $message . '</p></div>';
	}
}

echo nscan_display_tabs( 1 );
if ( nscan_is_valid() > 0 ) {
	$premium = '';
} else {
	$premium = ' <sup><font color="#D60404">'.
		__('Premium only', 'ninjascanner'). '</font></sup>';
}
?>
<form method="post">
	<?php
	wp_nonce_field('nscan_summary', 'nscannonce', 0);

	// Check and display scan status or the last scan date/time:
	nscan_get_blogtimezone();
	$scan_status = __('None', 'ninjascanner');
	if ( $nscan_is_running ) {
		// Currently running:
		$scan_status = '<div class="progress-bar"><span id="ns-span-progress" style="width:0%"></span><div id="ns-div-progress" class="progress-text">'. __('Please wait...', 'ninjascanner') .'</div></div>';


	} else {
		// Last scan date:
		if ( file_exists( NSCAN_SNAPSHOT ) ) {
			$ctime = filemtime( NSCAN_SNAPSHOT );
			if ( date( 'Y-m-d' ) == date( 'Y-m-d', $ctime ) ) {
				$scan_status = __('Today', 'ninjascanner') . date_i18n( ' @ g:i A', $ctime );
			} else {
				$scan_status = date_i18n( 'F d, Y @ g:i A', $ctime );
			}
		}
	}
	// Check for next scheduled scan:
	$ns_msg = __('None', 'ninjascanner');
	if ( $nextcron = wp_next_scheduled('nscan_scheduled_scan') ) {
		$sched = new DateTime( date( 'M d, Y H:i:s', $nextcron ) );
		$now = new DateTime( date( 'M d, Y H:i:s', time() ) );
		$diff = $now->diff( $sched );
		$day    = sprintf( _n( '%s day', '%s days', $diff->format('%a') % 7, 'ninjascanner' ), $diff->format('%a') % 7 );
		$hour   = sprintf( _n( '%s hour', '%s hours', $diff->format('%h'), 'ninjascanner' ), $diff->format('%h') );
		$minute = sprintf( _n( '%s minute', '%s minutes', $diff->format('%i'), 'ninjascanner' ), $diff->format('%i') );
		$second = sprintf( _n( '%s second', '%s seconds', $diff->format('%s'), 'ninjascanner' ), $diff->format('%s') );
		$ns_msg = sprintf( __('In approximately %s, %s, %s and %s.', 'ninjascanner'), $day , $hour, $minute, $second );
	}

	if (! empty($_POST['view-report'] ) ) {
		// Don't display the table header if we are viewing a scan report:
		$style = " style='display:none'";
	} else {
		$style = "";
	}
	?>
	<table class="form-table"<?php echo $style?>>
		<tr>
			<th scope="row"><?php _e('Last scan', 'ninjascanner') ?></th>
			<td><?php
				echo '<p>' . $scan_status . '</p>';
				// Display a button to view last scan report only
				// if a scan is not currenly running:
				if ( file_exists( NSCAN_SNAPSHOT ) && ! $nscan_is_running ) {
				?>
					<p><input class="button-secondary" name="view-report" value="<?php _e('View Scan Report', 'ninjascanner') ?>" type="submit"></p>
				<?php
				}
			?>
			</td>
		</tr>
		<tr>
			<th scope="row"><?php _e('Next scheduled scan', 'ninjascanner') ?></th>
			<td><?php echo $ns_msg; echo $premium; ?></td>
		</tr>
	</table>

	<br />

	<?php
	if ( empty( $nscan_is_running ) ) {
	?>
		<input type="submit" id="start-scan" name="start-scan" class="button-primary" value="<?php _e('Scan Your Blog', 'ninjascanner') ?> »"<?php echo $disable_scan ?> />
		&nbsp;&nbsp;&nbsp;&nbsp;
		<input type="submit" id="cancel-scan" name="cancel-scan" class="button-secondary" value="<?php _e('Cancel Scan', 'ninjascanner') ?>" disabled="disabled" />
	<?php
	} else {
	?>
		<input type="submit" id="start-scan" name="start-scan" class="button-primary" value="<?php _e('Scan Your Blog', 'ninjascanner') ?> »" disabled="disabled" />
		&nbsp;&nbsp;&nbsp;&nbsp;
		<input type="submit" id="cancel-scan" name="cancel-scan" class="button-secondary" value="<?php _e('Cancel Scan', 'ninjascanner') ?>"<?php echo $disable_scan ?> />
	<?php
	}
?>
</form>
<?php

// Don't delete (see file_quarantine.php):
echo '<!-- NinjaScanner Quarantine -->';

if (! empty( $report['body'] ) && empty( $report['error'] ) ) {
	echo "<br /><hr>";
	echo $report['body'];
}

// =====================================================================
// EOF
