/*
 +=====================================================================+
 |     _   _ _        _       ____                                     |
 |    | \ | (_)_ __  (_) __ _/ ___|  ___ __ _ _ __  _ __   ___ _ __    |
 |    |  \| | | '_ \ | |/ _` \___ \ / __/ _` | '_ \| '_ \ / _ \ '__|   |
 |    | |\  | | | | || | (_| |___) | (_| (_| | | | | | | |  __/ |      |
 |    |_| \_|_|_| |_|/ |\__,_|____/ \___\__,_|_| |_|_| |_|\___|_|      |
 |                 |__/                                                |
 |                                                                     |
 | (c) NinTechNet ~ https://nintechnet.com/                            |
 +=====================================================================+ // sa+i18n
*/

// Scan report:

function nscan_file_info( what, id ) {

	if (! what ) { return; }

	// Warn if the file was deleted since last scan:
	if ( what == 'deleted' ) {
		nscan_file_deleted();
		jQuery('#select-report-'+ id ).prop('selectedIndex',-1);
		return;
	}

	// Because we use a "multiple" select for aesthetic purposes
	// but don't want the user to select multiple files, we focus
	// only on the currently selected one:
	var current_item = jQuery('#select-report-'+ id ).prop('selectedIndex');
	jQuery('#select-report-'+ id ).prop('selectedIndex',current_item);

	// Retrieve data to display:
	// filename 0, uid 1, gid 2, size 3, mtime 4, ctime 5,
	//  mode 6, and optional data 7
	nscan_stat = what.split('::');

	// Show the file information:
	if ( id > 0 || id < 17 ) {
		if ( jQuery('#table-report-'+id).css('display') == 'none' ) {
			jQuery('#table-report-'+id).slideDown();
		}
		jQuery('#table-report-'+id+'-size').html(nscan_stat[3]);
		jQuery('#table-report-'+id+'-mode').html(nscan_stat[6]);
		jQuery('#table-report-'+id+'-uidgid').html(nscan_stat[1] +' / '+ nscan_stat[2]);
		jQuery('#table-report-'+id+'-mtime').html(nscan_stat[4]);
		jQuery('#table-report-'+id+'-ctime').html(nscan_stat[5]);
	}

	if ( id == 11 ) {
		// Antimalware scan: display the reason why the file
		// was tagged as suspicious:
		jQuery('#table-report-'+id+'-lmd').html(nscan_stat[7]);
	}
	if ( id == 3 ) {
		// Mismatch plugin/theme: store its slug and version:
		jQuery('#table-report-'+id+'-slug-version').html(nscan_stat[7]);
	}
	if ( id == 8 ) {
		// Binary file: display its format:
		jQuery('#table-report-'+id+'-bin').html(nscan_stat[7]);
	}

	// Add its name:
	jQuery('#table-report-'+id+'-fname').val(nscan_stat[0]);
	// Full value, used by quarantine and restore:
	jQuery('#table-report-'+id+'-value').val(what);

}

function nscan_post_info( what, id ) {

	if (! what ) { return; }

	// Warn if the post was deleted since last scan:
	if ( what == 'deleted' ) {
		nscan_post_deleted();
		jQuery('#select-report-'+ id ).prop('selectedIndex',-1);
		return;
	}

	// Because we use a "multiple" select for aesthetic purposes
	// but don't want the user to select multiple files, we focus
	// only on the currently selected one:
	var current_item = jQuery('#select-report-'+ id ).prop('selectedIndex');
	jQuery('#select-report-'+ id ).prop('selectedIndex',current_item);

	// Retrieve data to display:
	// post_id 0, post_type 1
	nscan_stat = what.split('::');

	if ( jQuery('#table-report-'+id).css('display') == 'none' ) {
		jQuery('#table-report-'+id).slideDown();
	}

	// Show post/page info:
	jQuery('#table-report-'+id+'-id').html(nscan_stat[0]);
	var post_type;
	if ( nscan_stat[1] == 'post' ) {
		post_type = nscani18n.post;
	} else {
		post_type = nscani18n.page;
	}

	jQuery('#table-report-'+id+'-type').html(post_type);

	// Add its ID (used by "View post"):
	jQuery('#table-report-'+id+'-value').val(nscan_stat[0]);

}

function nscan_file_deleted() {
	// Deleted file, no info available:
	alert( nscani18n.deleted_file );
}

function nscan_post_deleted() {
	// Deleted post, no info available:
	alert( nscani18n.deleted_post );
}

function nscan_close_info( id ) {

	jQuery('#table-report-'+ id ).slideUp();
	jQuery('#select-report-'+ id ).prop('selectedIndex',-1);

}

// View, compare, restore, quarantine files:
function nscan_op( file, what, nonce, id, val ) {

	// Already quarantined item?
	if ( val == 'deleted' ) {
		nscan_file_deleted();
		if ( id ) {
			jQuery('#select-report-'+ id ).prop('selectedIndex',-1);
		}
		return false;
	}

	// View file or compare files:
	if ( what == 'view' || what == 'compare' ) {

		// Note: "file" is already base64-encoded.
		var url = "?page=NinjaScanner&nscanop="+ what +"&file="+ encodeURIComponent( file )
					+"&nscanop_nonce="+ nonce;

		if ( what == 'view' ) {
			var signature = val.split('::');
			if (typeof signature[7] !== 'undefined') {
				url += '&signature=' + encodeURIComponent( signature[7] );
			}
		}

		height = 550;
		width = 900;
		var str = "height="+ height +",innerHeight="+ height +",width="+ width +",innerWidth="+ width;
		var ah = screen.availHeight - 30;
		var aw = screen.availWidth -10;
		var xc = (aw - width) / 2;
		var yc = (ah - height) / 2;
		str += ",left="+ xc +",screenX="+ xc +",top="+ yc +",screenY="+ yc
				+",scrollbars=yes,status=no,location=no,resizable=yes";
		win = open( url, "nscanpop", str );
		setTimeout("win.window.focus()",300);


	// Move the file to the quarantine folder:
	} else if ( what == 'quarantine' ) {

		if (! confirm( nscani18n.quarantine_confirm ) ) {
			return false;
		}
		// Show animated gif:
		jQuery('#quarantine-btn-'+ id).hide();
		jQuery('#quarantine-img-'+ id).show();

		var data = {
			'action': 'nscan_quarantine',
			'nscanop_nonce': nonce,
			'file': file
		};
		jQuery.post(ajaxurl, data, function(response) {
			if ( response == 'success' ) {
				// Mark it as quarantined:
				var cv = jQuery('#select-report-'+ id +' option[value="'+ val +'"]').html();
				jQuery('#select-report-'+ id +' option[value="'+ val +'"]').html(cv + ' '+ nscani18n.quarantined);
				jQuery('#select-report-'+ id +' option[value="'+ val +'"]').css('color','#9F9F9F');
				// Clear it:
				jQuery('#table-report-'+ id +'-value').val('deleted');
				jQuery('#select-report-'+ id +' option[value="'+ val +'"]').val('deleted');
				// Unselect the file:
				jQuery('#select-report-'+ id ).prop('selectedIndex',-1);

				alert( nscani18n.quarantine_success );

				// Close the table:
				jQuery('#table-report-'+ id ).slideUp();

			} else {
				alert( response );
			}

			// Hide animated gif:
			jQuery('#quarantine-btn-'+ id).show();
			jQuery('#quarantine-img-'+ id).hide();

		});

	// Restore the original file (core, plugin or theme):
	} else if ( what == 'restore' ) {

		if (! confirm( nscani18n.restore_confirm ) ) {
			return false;
		}
		// Show animated gif:
		jQuery('#restore-btn-'+ id).hide();
		jQuery('#restore-img-'+ id).show();

		var data = {
			'action': 'nscan_restore',
			'nscanop_nonce': nonce,
			'file': file
		};
		jQuery.post(ajaxurl, data, function(response) {
			if ( response == 'success' ) {
				// Mark it as restored:
				var cv = jQuery('#select-report-'+ id +' option[value="'+ val +'"]').html();
				jQuery('#select-report-'+ id +' option[value="'+ val +'"]').html(cv + ' '+ nscani18n.restored);
				jQuery('#select-report-'+ id +' option[value="'+ val +'"]').css('color','#9F9F9F');
				// Clear it:
				jQuery('#table-report-'+ id +'-value').val('deleted');
				jQuery('#select-report-'+ id +' option[value="'+ val +'"]').val('deleted');
				// Unselect the file:
				jQuery('#select-report-'+ id ).prop('selectedIndex',-1);

				alert( nscani18n.restore_success );

				// Close the table:
				jQuery('#table-report-'+ id ).slideUp();

			} else {
				alert( response );
			}

			// Hide animated gif:
			jQuery('#restore-btn-'+ id).show();
			jQuery('#restore-img-'+ id).hide();

		});

	} else {
		alert( nscani18n.unknown_action );
	}

}

// View post/pages:
function nscan_op_post( post_id ) {

	var url = 'post.php?post=' + post_id + '&action=edit';
	height = 550;
	width = 900;
	var str = "height="+ height +",innerHeight="+ height +",width="+ width +",innerWidth="+ width;
	var ah = screen.availHeight - 30;
	var aw = screen.availWidth -10;
	var xc = (aw - width) / 2;
	var yc = (ah - height) / 2;
	str += ",left="+ xc +",screenX="+ xc +",top="+ yc +",screenY="+ yc
			+",scrollbars=yes,status=no,location=no,resizable=yes";
	win = open( url, "nscanpop", str );
	setTimeout("win.window.focus()",300);

}

// =====================================================================

// Settings page:

function toggle_settings(what) {
	if ( what == 1) {
		jQuery("#nscan-advanced-settings").slideDown();
		jQuery("#nscan-show-advanced-settings").hide();
		jQuery("#nscan-show-nerds-settings").show();
	} else {
		jQuery("#nscan-nerds-settings").slideDown();
		jQuery("#nscan-show-nerds-settings").hide();
	}
}

function slow_scan_disable(what) {
	if ( document.getElementById(what).checked == false ) {
		if ( confirm( nscani18n.slow_down_scan_disable ) ) {
			return true;
		}
		return false;
	}
}
function slow_scan_enable(what) {
	if ( document.getElementById(what).checked == true ) {
		if ( confirm( nscani18n.slow_down_scan_enable ) ) {
			return true;
		}
		return false;
	}
}
function force_restart_enable(what) {
	if ( document.getElementById(what).checked == true ) {
		if ( confirm( nscani18n.force_restart_enable ) ) {
			return true;
		}
		return false;
	}
}

function restore_settings() {
	if ( confirm( nscani18n.restore_settings ) ) {
		return true;
	}
	return false;
}

function clear_cache() {
	if ( confirm( nscani18n.clear_cache_now ) ) {
		return true;
	}
	return false;
}

function ns_gsb_check_key( apikey, nonce ) {

	if (! apikey ) {
		alert( nscani18n.empty_apikey );
		jQuery('#nsgsb').focus();
		return false;
	}
	if (! nonce ) {
		return false;
	}

	jQuery('#nsgsb-button').hide();
	jQuery('#nsgsb-gif').show();

	var data = {
		'action': 'nscan_checkapikey',
		'nscanop_nonce': nonce,
		'api_key': apikey
	};
	jQuery.post(ajaxurl, data, function(response) {

		if ( response == 'success' ) {
			alert( nscani18n.success_apikey );

		} else {
			alert( response );
			jQuery('#nsgsb').select();
		}

		jQuery('#nsgsb-gif').hide();
		jQuery('#nsgsb-button').show();

	});
	return true;
}

// =====================================================================

// Quarantine page:

function quarantine_form(what) {

	if ( document.getElementById('qf').selectedIndex == -1 ) {
		alert( nscani18n.select_elements );
		return false;
	}

	// Permanently delete files:
	if ( what == 1 ) {
		if ( confirm( nscani18n.permanently_delete ) ) {
			return true;
		}
	// Restore quarantined files:
	} else {
		if ( confirm( nscani18n.restore_file ) ) {
			return true;
		}
	}

	return false;
}

// =====================================================================

// Log page:

function nscan_filter_log() {

	// Create bitmask:
	var bitmask = 0;
	if ( document.nscanlogform.info.checked == true ) { bitmask += 1; }
	if ( document.nscanlogform.warn.checked == true ) { bitmask += 2; }
	if ( document.nscanlogform.error.checked == true ) { bitmask += 4; }
	if ( document.nscanlogform.debug.checked == true ) { bitmask += 8; }

	// Clear the textarea:
	document.nscanlogform.nscantxtlog.value = '';

	// Browser through our array and return only selected verbosity:
	var nscan_count = 0;
	for ( i = 0; i < nscan_array.length; ++i ) {
		var line = decodeURIComponent( nscan_array[i] );
		var line_array = line.split( '~~', 2 );
		if ( line_array[0] & bitmask ) {
			document.nscanlogform.nscantxtlog.value += line_array[1];
			++nscan_count;
		}
	}
	if ( nscan_count == 0 ) {
		document.nscanlogform.nscantxtlog.value = '\n  > ' + nscani18n.empty_log;
	}
}

// =====================================================================
// Highlight code.

function nscan_highlight() {

	var nscan_content = document.getElementById('nscan-highlight').innerHTML;
	nscan_content = nscan_content.replace(/NSCANFOO/g, '<font style="background-color:yellow">');
	nscan_content = nscan_content.replace(/NSCANBAR/g, '</font>');
	document.getElementById('nscan-highlight').innerHTML = nscan_content;

}

// =====================================================================
// EOF
